function [p,L,U]=pLU_Vec(A)
% permuted LU factorization for 3D-array
%   P*A=L*U
% p is the index of permutations for each matrices.
% One can use P=fc_linalg3D.fact.PermMatFromArray(p) to build the
% N-by-d-by-d 3D-array permutation matrices.
  assert( fc_linalg3D.is3Darray(A) )
  [N,m,n]=size(A);
  assert( m==n )
  L=fc_linalg3D.eye(N,m);
  U=A;
  p=repmat(1:m,N,1); % N-by-m permutations indices
  K=1:N;J=ones(1,N);
  Subp=@(K,I) sub2ind([N,m],K,I);
  Subs=@(K,I,J) sub2ind([N,m,m],K,I,J);
  KK=[K,K];
  
  if ~fc_tools.comp.isOctave() && verLessThan('matlab','9.1.0')
    redim=@(A,ncols) repmat(A,[1 1 ncols ]);
  else
    redim=@(A,ncols) A;
  end
  
  for i=1:m
    [~,idx]=max(abs(U(:,i:end,i)),[],2);
    mu=idx'+i-1;
    l=m-i+1;
    mu=idx'+i-1;iJmu=[i*J,mu];muiJ=[mu,i*J];
    p(Subp(KK,iJmu))=p(Subp(KK,muiJ));
    for j=i:m
      U(Subs(KK,iJmu,[j*J,j*J]))=U(Subs(KK,muiJ,[j*J,j*J]));
    end
    for j=1:i-1
      L(Subs(KK,iJmu,[j*J,j*J]))=L(Subs(KK,muiJ,[j*J,j*J]));
    end   
    I=abs(U(:,i,i))>1e-15;
    for j=i+1:m
      L(I,j,i)=U(I,j,i)./U(I,i,i);
      cols=i:n;
      U(I,j,cols)=U(I,j,cols)-redim(L(I,j,i),length(cols)).*U(I,i,cols);
    end
  end  
  if nargout==1, p={p,L,U};end
end
